* Coll3: fodder for Collections Part 3 article:
CLEAR
#IF .F.
ON KEY LABEL F12 KEYBOARD "#IF .F." + CHR(13) + CHR(10) + "#ENDIF" + CHR(13) + CHR(10)
#ENDIF


* Example 1.a: Tolerate redundant keys by ignoring. [Discuss use case...]
loObj = CREATEOBJECT("IgnoreDupsCollection")
loObj.Add(DATETIME(), "abc")
loObj.Add(DATETIME(), "abc") && should be ignored, instead of error
? Heading(loObj)
loObj.ListKeys() && prints: 1 abc
* Example 1.b: Tolerate redundant keys by ignoring via TRY. [Discuss use case...]
loObj = CREATEOBJECT("Collection_1b")
loObj.Add(DATETIME(), "abc")
loObj.Add(DATETIME(), "abc") && should be ignored, instead of error
? Heading(loObj)
loObj.ListKeys() && prints: 1 abc

* compare 1a and 1b for performance
lnIter = 10000
lnHitRate = 0.01 && try/catch is slower unless error rate is very low
for jj = 1 to 2
  lnStart = seconds()
  lcClass = "Collection_1" + chr(asc("a") + m.jj - 1)
  loObj = createobject(m.lcClass)
  loObj.Add(0, "0")
  for ii = 1 to lnIter
    loObj.Add(m.ii, transform(iif(rand()<m.lnHitRate, 0, m.ii)))
  endfor
  ? "Class " + m.lcClass, seconds() - m.lnStart
endfor
*
CLEAR ALL
RETURN

DEFINE CLASS BaseCollection as Collection
  Length = NULL

  FUNCTION Length_ACCESS
    RETURN this.Count
  ENDFUNC 

  FUNCTION Reset
    THIS.Remove(-1)
  ENDFUNC 

  FUNCTION Replace(tvItem, tcKey)
    IF THIS.GetKey(m.tcKey) > 0 
      THIS.Remove(m.tcKey)
    ENDIF
    THIS.Add(m.tvItem, m.tcKey)
  ENDFUNC

  FUNCTION ListKeys
    LOCAL ii
    FOR ii = 1 TO THIS.Count
      ? m.ii, this.GetKey(m.ii), this.Item(m.ii)
    ENDFOR
  ENDFUNC  
ENDDEFINE && BaseCollection

DEFINE CLASS DataCollection as Collection
  cKeyExpr = NULL
  FUNCTION Item(tvIndex)
    IF THIS.GetKey(m.tvIndex) = 0
      * item not in collection
      LOCAL lcExpr
      lcExpr = THIS.cKeyExpr + ;
        [='] + m.tvIndex + [']
      LOCATE FOR &lcExpr
      IF FOUND()
        LOCAL loObj
        SCATTER MEMO NAME loObj
        THIS.Add(m.loObj, m.tvIndex)
        RETURN m.loObj
      ELSE
        ERROR 2061  && what they deserve
      ENDIF
    ENDIF 
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC
ENDDEFINE

DEFINE CLASS CaseInsensitiveCollection as BaseCollection
  FUNCTION add(tvItem, tcKey, tvBefore, tvAfter)
    NODEFAULT 
    DO CASE 
    CASE PCOUNT() = 1
      DODEFAULT(m.tvItem)
    CASE PCOUNT() = 2
      DODEFAULT(m.tvItem, UPPER(m.tcKey))
    CASE PCOUNT() = 3
      * Need to convert before to upper case also:
      tvBefore = IIF(VARTYPE(m.tvBefore)="C", ;
        UPPER(m.tvBefore), m.tvBefore)
      DODEFAULT(m.tvItem, UPPER(m.tcKey), m.tvBefore)
    CASE PCOUNT() = 4
      * Need to convert after to upper case also:
      tvAfter = IIF(VARTYPE(m.tvAfter)="C", ;
        UPPER(m.tvAfter), m.tvAfter)
      DODEFAULT(m.tvItem, UPPER(m.tcKey), , m.tvAfter)
      * This only works if you pass a blank 3rd parameter,
      * thus the need for 4 explicit CASE statements!
    ENDCASE 
  ENDFUNC 
  
  FUNCTION item(tvIndex)
    NODEFAULT
    IF VARTYPE(m.tvIndex) = "C"
      RETURN DODEFAULT(UPPER(m.tvIndex))
    ELSE 
      RETURN DODEFAULT(m.tvIndex)
    ENDIF 
  ENDFUNC 
  
  FUNCTION remove(tvIndex)
    NODEFAULT 
    IF VARTYPE(m.tvIndex) = "C"
      RETURN DODEFAULT(UPPER(m.tvIndex))
    ELSE 
      RETURN DODEFAULT(m.tvIndex)
    ENDIF 
  ENDFUNC 

  FUNCTION getKey(tvIndex)
    NODEFAULT 
    IF VARTYPE(m.tvIndex) = "C"
      RETURN DODEFAULT(UPPER(m.tvIndex))
    ELSE 
      RETURN DODEFAULT(m.tvIndex)
    ENDIF 
  ENDFUNC 
ENDDEFINE  && CaseInsensitiveCollection 

DEFINE CLASS IgnoreDupsCollection as Collection
  FUNCTION Add(tvItem, tcKey)
    NODEFAULT
    IF THIS.GetKey(m.tcKey) > 0 
      * key exists, ignore request
    ELSE
      RETURN DODEFAULT(m.tvItem, m.tcKey)
    ENDIF
  ENDFUNC
ENDDEFINE

DEFINE CLASS ReplaceDupsCollection as Collection
  FUNCTION Add(tvItem, tcKey)
    NODEFAULT
    IF THIS.GetKey(m.tcKey) > 0 
      * key exists
      THIS.Remove(m.tcKey)
    ENDIF
    RETURN DODEFAULT(m.tvItem, m.tcKey)
  ENDFUNC
  FUNCTION Replace(tvItem, tcKey)
    IF THIS.GetKey(m.tcKey) > 0 
      THIS.Remove(m.tcKey)
    ENDIF
    THIS.Add(m.tvItem, m.tcKey)
  ENDFUNC
ENDDEFINE

DEFINE CLASS ComboCollection as BaseCollection
  * Add optional case-sensitivity and collision handling
  * to the BaseCollection class.
  lCaseSensitive = .F. && set .T. to enable
  cOnCollision = "Error" && choose from {Error, Ignore, Replace}
  
  FUNCTION add(tvItem, tcKey, tvBefore, tvAfter)
    NODEFAULT 
    IF THIS.lCaseSensitive
      tvKey = IIF(VARTYPE(m.tvKey) = "C", ;
        UPPER(m.tvKey), m.tvKey)
      tvBefore = IIF(VARTYPE(m.tvBefore)="C", ;
        UPPER(m.tvBefore), m.tvBefore)
      tvAfter = IIF(VARTYPE(m.tvAfter)="C", ;
        UPPER(m.tvAfter), m.tvAfter)
    ENDIF
    
    IF PCOUNT() > 1 AND THIS.GetKey(m.tcKey) > 0 
      * key exists
      DO CASE
      CASE THIS.cOnCollision = "Ignore" 
        RETURN 
      CASE THIS.cOnCollision = "Replace" 
        THIS.Remove(m.tcKey)
        * then, let all the other before/after stuff happen below
      OTHERWISE && presume "Error" 
        ERROR 2062  && simulate same error they would have received
      ENDCASE
      RETURN
    ENDIF

    DO CASE 
    CASE PCOUNT() = 1
      DODEFAULT(m.tvItem)
    CASE PCOUNT() = 2
      DODEFAULT(m.tvItem, UPPER(m.tcKey))
    CASE PCOUNT() = 3
      DODEFAULT(m.tvItem, UPPER(m.tcKey), m.tvBefore)
    CASE PCOUNT() = 4
      DODEFAULT(m.tvItem, UPPER(m.tcKey), , m.tvAfter)
      * This only works if you pass a blank 3rd parameter,
      * thus the need for 4 explicit CASE statements!
    ENDCASE 
  ENDFUNC 
  
  FUNCTION item(tvIndex)
    IF THIS.lCaseSensitive AND VARTYPE(m.tvIndex) = "C"
      NODEFAULT
      tvIndex = UPPER(m.tvIndex)
    ENDIF
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC 
  
  FUNCTION remove(tvIndex)
    IF THIS.lCaseSensitive AND VARTYPE(m.tvIndex) = "C"
      NODEFAULT 
      tvIndex = UPPER(m.tvIndex)
    ENDIF 
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC 

  FUNCTION getKey(tvIndex)
    IF THIS.lCaseSensitive AND VARTYPE(m.tvIndex) = "C"
      NODEFAULT 
      tvIndex = UPPER(m.tvIndex)
    ENDIF 
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC 
ENDDEFINE  && ComboCollection

DEFINE CLASS JITCollection as BaseCollection
  cItemClass = NULL  && you provide
  FUNCTION Item(tvIndex)
    IF VARTYPE(m.tvIndex) = "C"
      LOCAL lnKey, loObj
      lnKey = THIS.GetKey(m.tvIndex)
      IF m.lnKey = 0 && d.n.e. 
        NODEFAULT 
        loObj = CREATEOBJECT(THIS.cItemClass)
        THIS.Add( m.loObj, m.tvIndex) 
        RETURN m.loObj
      ENDIF
    ENDIF 
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC 
ENDDEFINE 

DEFINE CLASS DateKeyCollection as BaseCollection
  FUNCTION Add(tvItem, tvKey)
    NODEFAULT 
    IF VARTYPE(m.tvKey) = "D"
      tvKey = DTOS(m.tvKey)
    ENDIF 
    DODEFAULT(m.tvItem, m.tvKey)
  ENDFUNC 
* Some failed attempts for reader amusement:
  *!*    FUNCTION Item(tvIndex)
  *!*      LOCAL lcIndex
  *!*      IF VARTYPE(m.tvIndex) = "D"
  *!*        lcIndex = DTOS(m.tvIndex)
  *!*      ELSE
  *!*        lcIndex = m.tvIndex
  *!*      ENDIF 
  *!*      * This fails when date key passed: 
  *!*      RETURN DODEFAULT(m.lcIndex)
  *!*    ENDFUNC 
  *!*    FUNCTION Item(tvIndex)
  *!*      LOCAL lnKey
  *!*      IF VARTYPE(m.tvIndex) $ "IN"
  *!*        lnKey = m.tvIndex
  *!*      ELSE
  *!*        LOCAL lcKey
  *!*        IF VARTYPE(m.tvIndex) = "D"
  *!*          lcKey = DTOS(m.tvIndex)
  *!*        ELSE 
  *!*          lcKey = m.tvIndex
  *!*        ENDIF
  *!*        lnKey = THIS.GetKey(m.lcKey)
  *!*      ENDIF 
  *!*      RETURN DODEFAULT(m.lnKey)
  *!*    ENDFUNC
* And one that works:
  FUNCTION Item(tvIndex)
    IF VARTYPE(m.tvIndex) = "D"
      NODEFAULT 
      tvIndex = DTOS(m.tvIndex)
    ENDIF
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC
  
  FUNCTION GetKey(tvIndex)
    IF VARTYPE(m.tvIndex) = "D"
      NODEFAULT 
      tvIndex = DTOS(m.tvIndex)
    ENDIF
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC

  FUNCTION Remove(tvIndex)
    NODEFAULT  
    IF VARTYPE(m.tvIndex) = "D"
      tvIndex = DTOS(m.tvIndex)
    ENDIF 
    RETURN DODEFAULT(m.tvIndex)
  ENDFUNC 
ENDDEFINE 

DEFINE CLASS Collection_1b as BaseCollection
  FUNCTION Add(tvItem, tcKey)
    NODEFAULT 
    TRY  
      DODEFAULT(m.tvItem, m.tcKey)
    CATCH TO loExc WHEN loExc.ErrorNo = 2062
      = .f. && ignore
    ENDTRY 
  ENDFUNC
ENDDEFINE

DEFINE CLASS DayWorker AS Custom
  ADD OBJECT Events as Collection
ENDDEFINE 

* Silly UDF:
FUNCTION Heading(loObj)
  RETURN "Example #" + GETWORDNUM(loObj.Class, 2, "_")
ENDFUNC  
