*-- Open a project to iterate
MODIFY PROJECT test NOPROJECTHOOK NOWAIT

*-- Create a timer that will perform the image capture
poCaptureTimer=CREATEOBJECT("CaptureTimer")

LOCAL lnFiles, lnFile, lcExt, lnLines, lcFile
LOCAL ARRAY laClasses[1]

*-- Create a cursor to hold the list of forms found
CREATE CURSOR FormList( cFormName C(254), cSource C(5),mLibrary M, lExclude L)

lnFiles = _VFP.ActiveProject.Files.Count

*-- Loop through the files in the project
FOR lnFile = 1 TO lnFiles
	loFile = _VFP.ActiveProject.Files(lnFile)

	*-- Get the extension of the file in order to
	*-- determine how to find forms within the type
	
	lcExt = UPPER(SUBSTRC(loFile.Name,RAT(".",loFile.Name)+1))
	
	DO CASE
		CASE lcExt = "VCX"
			lnClasses = AVCXCLASSES(laClasses,loFile.Name)
			*-- Loop through classes looking for a baseclass
			*-- of "form" 
			FOR lnClass = 1 TO lnClasses
				IF UPPER(laClasses[lnClass,2])="FORM"
					
					INSERT INTO FormList ;
						(cFormName, cSource,mLibrary) ;
						VALUES ;
						(laClasses[lnClass,1],lcExt, loFile.Name)
				ENDIF
			ENDFOR

		CASE lcExt = "PRG"
			*-- Loop through lines of code looking for
			*-- lines beginning with DEFINE CLASS, find the
			*-- "AS" clause and see if its a form
			lcFile = FILETOSTR(loFile.Name)
			lnLines = ALINES(laLines,lcFile)
			
			FOR lnLine = 1 TO lnLines
				lcLine = ALLTRIM(UPPER(laLines[lnLine]))
				IF "DEFINECLASS" $ STRTRAN(lcLine," ","")
					IF RIGHTC(lcLine,5) =" FORM"
						lcName = SUBSTRC(lcLine,ATCC("CLASS",lcLine))
						lcName = ALLTRIM(SUBSTRC(lcName,6))
						lcName = SUBSTRC(lcName,1,AT(" ",lcName)-1)
						INSERT INTO FormList ;
							(cFormName, cSource, mLibrary) ;
							VALUES ;
							(lcName,lcExt, loFile.Name)
					ENDIF
				ENDIF
			ENDFOR
			
		CASE lcExt = "SCX"
			*-- If its a form, add the name of the
			*-- form to the list
			INSERT INTO FormList ;
				(cFormName, cSource) ;
			VALUES ;
				(loFile.Name,lcExt)

		CASE lcExt = ""
	ENDCASE
ENDFOR

*-- Give an opportunity to mark forms as excluded
BROWSE

GO TOP
SCAN FOR !lExclude
	*-- run the form, but dont show it yet
	DO CASE
		CASE cSource = "SCX"
			DO FORM (ALLTRIM(cFormName)) NOSHOW NAME loForm LINKED 
			
		CASE cSource = "VCX" OR cSource = "PRG" 
			loForm = NEWOBJECT(ALLTRIM(cFormName),ALLTRIM(MLibrary))
	ENDCASE

	*-- Make the form modal to create a wait state
	loForm.WindowType= 1
	*-- Pass a reference to the form to the
	*-- timer that will capture the form
	poCaptureTimer.oForm = loForm
	*-- Enable the timer
	poCaptureTimer.Enabled=.T.
	*-- Show the form
	loForm.Show()
ENDSCAN

GO Top
STRTOFILE("<HTML><BODY><H1>Quick Manual</H1><BR><BR>","quickmanual.htm")
SCAN 
	lcFormName = ALLTRIM(cFormName)
	IF "\"$cFormName
		lcFormName = SUBSTRC(lcFormName,RATC("\",lcFormName)+1)
		lcFormName = SUBSTRC(lcFormName,1,RATC(".SCX",UPPER(lcFormName))-1)
	ENDIF
	STRTOFILE("<H2>"+ALLTRIM(lcFormName)+"</H2>","quickmanual.htm",1)
	STRTOFILE([<img src="]+SYS(5)+CURDIR()+[capturedimages\]+ALLTRIM(lcFormName)+[.png">],"quickmanual.htm",1)
ENDSCAN
STRTOFILE("</BODY></HTML>","quickmanual.htm",1)
USE IN SELECT("formlist")

_vfp.ActiveProject.Close()

loShellExec = NEWOBJECT("_shellexecute",HOME()+"ffc\_environ")
loShellExec.ShellExecute(SYS(5)+CURDIR()+"quickmanual.htm")

*-- open explorer to capturedimages directory
lcExecute = "! explorer "+ SYS(5)+CURDIR()+"capturedimages"
&lcExecute


DEFINE CLASS CaptureTimer AS Timer
Interval = 500
oForm = .NULL.
Enabled = .F.

PROCEDURE Timer
	this.Enabled = .F.
	
	this.Capture()
	
	LOCAL loForm
	loForm = this.oForm
	this.oForm=.NULL.
	
	*-- Release the form, ending the
	*-- wait state
	loForm.Release()
		
ENDPROC

PROCEDURE Capture
	*-- Capture the form's image
	LOCAL loImageCapture AS SnagIt.ImageCapture
	loImageCapture = CreateObject("SnagIt.ImageCapture")
	*loImageCapture.IncludeCursor=.F.
	loImageCapture.Input=1 && siiWindow
	loImageCapture.Output= 2  && sioFile 

	loImageCapture.InputWindowOptions.SelectionMethod = 2 && swsmHandle
	loImageCapture.InputWindowOptions.Handle          = this.oform.HWnd

	*-- Store it to a PNG file, named after the form
	loImageCapture.OutputImageFile.FileNamingMethod = 1 && sofnmFixed
	loImageCapture.OutputImageFile.FileType         = 5 && siftPNG
	loImageCapture.OutputImageFile.Directory        = SYS(5)+CURDIR()+"\capturedimages"
	IF "\"$cFormName
		lcImage = ALLTRIM(cFormName)
		lcImage = SUBSTRC(lcImage,RATC("\",lcImage)+1)
		lcImage = SUBSTRC(lcImage,1,RATC(".",lcImage)-1)
		
		loImageCapture.OutputImageFile.FileName = lcImage
	ELSE
		loImageCapture.OutputImageFile.FileName = ALLTRIM(cFormName)
	ENDIF
	loImageCapture.Capture()
ENDPROC

ENDDEFINE